cutSynmap<- function(projinfo) {

  synmap <- getr(paste("synmap.",synmaptouse,sep=""),rdatain) #created in gridEurope.r

  #first reverse
  synmap <- synmap[dim(synmap)[1]:1,]

  sely <- projinfo$lat.syn>=(projinfo$y0)&projinfo$lat.syn<=(projinfo$y1)
  selx <- projinfo$lon.syn>=(projinfo$x0)&projinfo$lon.syn<=(projinfo$x1)
  synmapn<-synmap[sely,selx]

  synmapn[synmapn==0]   <- 48 #add water!
  synmap2vprm.key       <- read.table(paste(scriptpath,"/RSources/synmap2vprm.txt",sep=""),header=TRUE)
  synmap2vprm           <- synmap2vprm.key[synmapn,"VPRM"]
  synmap2vprm           <- matrix(synmap2vprm,ncol=dim(synmapn)[2])
  dimnames(synmap2vprm) <- dimnames(synmapn)

  # save for later use
  assignr(paste("vegetationmapCutted.",projinfo$year,sep=""),synmap2vprm,datp)

  return(list(synmap2vprm,sely,selx))
}

getDomainMinMax<-function(vari,year,doyuse,synmap2vprm){
  # get minimum indices map
  # to do: take flexible snow times into account
  FINAL   <- getr(paste(vari,".indicesCutted.",year,".1",sep=""),datp)

  min <- FINAL

  for (tstep in 2:length(doyuse)) {
   FINAL  <- getr(paste(vari,".indicesCutted.",year,".",tstep,sep=""),datp)
   min    <- getMinMat(min,FINAL) #defined in RSources/getminmax.r
  }

  rm(FINAL);gc()

  return(min)
}

classIndicesInTimeStep<-function(vari,year,tstep,minima,water,synmap2vprm) {

  FINAL <- getr(paste(vari,".indicesCutted.",year,".",tstep,sep=""),datp)
  
  # replace the NA in winter with minimum indices map
  isna        <- is.na(FINAL)&!water
  FINAL[isna] <- minima[isna]

  # separate vegetation classes
  nveg           <- 8
  indicesClassed <- array(dim=c(dim(synmap2vprm)[1],dim(synmap2vprm)[2],nveg))
  vegs           <- which(1:8%in%synmap2vprm)
  for(v in vegs) {
    indicesClassed[,,v]<-FINAL
    indicesClassed[,,v][synmap2vprm!=v]=NA
  }
  assignr(paste(vari,".indicesClassed.",year,".",tstep,sep=""),indicesClassed,datp)

}

cutToDomainInTimeStep<-function(sely,selx,vari,synmap2vprm,tstep,year,doyuse) {

  # check if work was already done
  if (file.exists(paste(datp,"/.RData",vari,".indicesCutted.",year,".",tstep,sep=""))) {    
    print(paste(datp,"/.RData",vari,".indicesCutted.",year,".",tstep," exists.",sep=""))
    return(NULL)
  }

  # cut part out of combined tiles with same size as vegmap
  grid <- getr(paste(vari,".",year,".",tstep,".combined",sep=""),datp)
  grid <- grid[sely,selx]

  # gap filling with counting of NAs replaced
  water <- synmap2vprm==8
  water[is.na(water)] <- FALSE
  # count NAs over land
  nasbefore <- length(grid[is.na(grid)&!water])
  nasafter  <- nasbefore
  # replace at most 4 times
  i <- 0
  while (nasafter!=0&&i<4) {
   grid    <- na.replace(grid,diagTF=TRUE)
   # count NAs for land
   nasafter <- length(grid[is.na(grid)&!water])
   i <- i+1
  }
  nasafter  <- length(grid[is.na(grid)&!water])
  print("Summary of NA replacement (over land):")
  print(date())
  print(paste("Timestep: ",tstep,"; Variable: ",vari,sep=""))
  print("NAs before: ");print(nasbefore)
  print("NAs after: ");print(nasafter)
  
  # done. now save results:
  assignr(paste(vari,".indicesCutted.",year,".",tstep,sep=""),grid,datp)  
}

cutToDomain<-function(vari="evi",doyuse,projinfo) {

  skip <- TRUE
  for (tstep in 1:length(doyuse))
   if (!file.exists(paste(datp,"/.RData",vari,".indicesClassed.",projinfo$year,".",tstep,sep=""))) skip=FALSE

  if (!skip||!file.exists(paste(datp,"/.RDatavegetationmapCutted.",projinfo$year,sep=""))) {
   synmap      <- cutSynmap(projinfo)
   synmap2vprm <- synmap[[1]]
   sely        <- synmap[[2]]
   selx        <- synmap[[3]]
   rm(synmap);gc()
  }
  
  if (skip) {
   print(paste("Classed indices for ",toupper(vari)," already there. Skipping.",sep=""))
   return(NULL)
  }

  print(date())
  print(paste("Steps total: ",length(doyuse),sep=""))
  for (tstep in 1:length(doyuse))
   cutToDomainInTimeStep(sely,selx,vari,synmap2vprm,tstep,projinfo$year,doyuse)
   
  # get min/max and class indices; has to happen after cutting all time steps,
  # since min/max is calculated over whole year
  min <- getDomainMinMax(vari,projinfo$year,doyuse,synmap2vprm)

  # get watermask
  water <- synmap2vprm==8
  water[is.na(water)] <- FALSE

  # separate indices in different vegetation classed classes
  # result is written to separate file
  for (tstep in 1:length(doyuse))
   classIndicesInTimeStep(vari,projinfo$year,tstep,min,water,synmap2vprm)

  return(NULL)
}
