# Script for loess filtering of one MODIS tile, called by submit_loess.sh.
# Purpose is parallel execution on cluster via LSF queueing system.


scriptpath <- getwd() # path of preprocessor
sourcepath <-paste(scriptpath,"/RSources/",sep="")

source(paste(scriptpath,"/RSources/monthdayyear.r",sep=""))

# read configuration
source(paste(scriptpath,"/config.r",sep=""))

ftdata   <- FALSE          # shall we connect to ftp and check for data?
projtype <- "LCC"       # type of projection LCC xor GEO
datp <- rdataout
mdy      <- month.day.year
attachn  <- function(what, pos = 2){if(!(paste("file:",what,sep="")%in%search()))attach(what,pos)}
go       <- graphics.off

# set synmap resolution
spacing <- 1/120

# set number of used vegetation classes
nveg <- 8 

setwd(rdataout)

#library("hdf5")
library("h5r")

source(paste(scriptpath,"/RSources/assignr.r",sep=""))
source(paste(scriptpath,"/RSources/project.nice.r",sep=""))
source(paste(scriptpath,"/RSources/getminmax.r",sep=""))
source(paste(scriptpath,"/RSources/distance.r",sep=""))
source(paste(scriptpath,"/RSources/getr.r",sep=""))
source(paste(scriptpath,"/RSources/loess.r",sep=""))
source(paste(scriptpath,"/RSources/imagell.r",sep=""))
source(paste(scriptpath,"/RSources/na_replace.r",sep=""))
source(paste(scriptpath,"/RSources/combineTiles.r",sep=""))
source(paste(scriptpath,"/RSources/hdfread.r",sep=""))
source(paste(scriptpath,"/RSources/parseNamelist.r",sep=""))
source(paste(scriptpath,"/RSources/getModis.r",sep=""))
source(paste(scriptpath,"/RSources/extractAndReduce.r",sep=""))
source(paste(scriptpath,"/RSources/doResampling.r",sep=""))
source(paste(scriptpath,"/RSources/readIndicesIntoR.r",sep=""))
source(paste(scriptpath,"/RSources/cutToDomain.r",sep=""))
source(paste(scriptpath,"/RSources/doAggregation.r",sep=""))
source(paste(scriptpath,"/RSources/writeOutput.r",sep=""))

# load configuration
projinfo <- getr("projinfo",datp)

# set 8 day intervals to process
# to do: calculate from start and end date set in namelist!
doyuse <- doyall  <- seq(as.numeric(paste(projinfo$year,"001",sep="")),as.numeric(paste(projinfo$year,projinfo$ldoy,sep="")),by=8)

doyuse<-doyuse[doyuse!=2001169] #exclude missing MODIS acquisition period

# cd to working directory
setwd(mpath)

load("TLUT.RData")




args <- commandArgs()
print(args)

H    <- args[5]
V    <- args[6]
vari <- args[7]
projs<-paste("h",H,"v",V,".",projinfo$year,".ier1km",sep="")

print(projs)


wait_option<-0
if  (length(args)>7) {
  wait_option<-1
  needed_jobs<-as.numeric(args[8])
  }

if (wait_option==0) { # normal run
Smoothevilswi(vari,proj=projs,doyall=doyall,doyuse=doyuse) # defined in loess.r

} else #wait_option==0
{ #wait_option==1
 print(paste("DEBUG: process_loess_wait.r with wait_option and needed_jobs:",needed_jobs,sep=""))
 finished_jobs<-0
 last_write_flag<-0
# print(paste("DEBUG start loop for pattern:","^.RData",vari,".smooth.",projs,sep=""))
 print(paste("DEBUG start loop for pattern:","^.RData",vari,".smooth.",sep=""))
 while (finished_jobs<needed_jobs) {
# file_list<-list.files(path=datp,pattern=paste("^.RData",vari,".smooth.",projs,sep="") ,all.files=TRUE,no..=TRUE,full.names=TRUE)
 file_list<-list.files(path=datp,pattern=paste("^.RData",vari,".smooth.",sep="") ,all.files=TRUE,no..=TRUE,full.names=TRUE)
 finished_jobs<-length(file_list)
 if (length(file_list)>=needed_jobs) {
    print("Now count the files with 'Successfully completed'")
   file_list<-list.files(path=".",pattern="process_loess.r.Rout." ,all.files=FALSE,no..=TRUE,full.names=TRUE)
   complete_counter=0
   for (i in 1:length(file_list)) {
    # grepl return a logical vector for each element in the file
    if (any(grepl("Successfully completed",readLines(file_list[i])))==TRUE) {complete_counter<-complete_counter+1}
    print(paste("DEBUG: i  complete_counter:",i,complete_counter))
    } # for
   if (complete_counter==length(file_list)) {
     print(paste("DEBUG: length(file_list)",length(file_list),sep=""))
     print(paste("DEBUG: complete_counter:",complete_counter,sep=""))
     print(paste("DEBUG: process_loess_wait.r will finished in 5 min domain:",dom,sep=""))
     break;
     }  #if complete_counter

#   details<-file.info(file_list)
#   # now sort according last modification if files 
#   details<-details[with(details,order(as.POSIXct(mtime))),]
#   file_list<-rownames(details)
#   #Now test, if for the last modififile was modification was more as 5 min ago
#   counts<-dim(details)[1] # counts of files
#   last_modification<-details[,"mtime"][counts]
#   #get actual time
#   act_time<-Sys.time()
#   time_diff<-as.numeric(difftime(act_time,last_modification,units="sec")) #in sec
#   if (time_diff>60*15) { # 15 min
#     last_write_flag=1 
#     print("DEBUG: last file modification was more as 5 min before")
#     print(paste("DEBUG: act_time:",act_time,sep=""))
#     print(paste("DEBUG: last_modification:",last_modification,sep=""))
#     print(paste("DEBUG: for File:",rownames(details)[counts],sep=""))
#     print(paste("DEBUG: process_loess_wait.r will finished in 5 min domain:",dom,sep=""))


    } # if length(file_list)
 Sys.sleep(60*15) # is 60 sec * 15 = 15min

 } # while
 Sys.sleep(60*5) # is 60 sec * 5 = 5min
print("DEBUG: process_loess_wait.r is now finished ok")
} #wait_option==1
