#create land-water mask from synmap

source("config.r")
source(paste(scriptpath,"/RSources/getr.r",sep=""))
datp     <- rdataout
projinfo <- getr("projinfo",datp)

synmap <- getr(paste("synmap.",synmaptouse,sep=""),rdatain) #created in gridEurope.r
  sely <- projinfo$lat.syn>=(projinfo$y0)&projinfo$lat.syn<=(projinfo$y1)
  selx <- projinfo$lon.syn>=(projinfo$x0)&projinfo$lon.syn<=(projinfo$x1)
  synmapn<-synmap[sely,selx]

land<-1*(synmapn>0)

#aggregate
# set synmap resolution
spacing <- 1/120
  xfactor <- projinfo$dxy/spacing
  yfactor <- projinfo$dyy/spacing

  mapAggregated <- matrix(nrow=projinfo$ny,ncol=projinfo$nx)
  for (y in 0:(dim(mapAggregated)[1]-1)) {
    for (x in 0:(dim(mapAggregated)[2]-1)) {
      mapAggregated[y+1,x+1] <- sum(land[(y*yfactor+1):(y*yfactor+yfactor),(x*xfactor+1):(x*xfactor+xfactor)],na.rm=T)
    }
  }

landn<-1*(mapAggregated>0.5*xfactor*yfactor)

landn<-landn[nrow(landn):1,] #reverse to get correct order

#write ncdf file
 
 # make sure that the needed library is loaded
 library(RNetCDF)
 
 # calculate coordinates
 lon <- seq(projinfo$x0,projinfo$x1,by=projinfo$dxy)[1:projinfo$nx]
 lat <- seq(projinfo$y0,projinfo$y1,by=projinfo$dyy)[1:projinfo$ny]

 # get data
 map<-t(landn)

 # create netcdf file: create.ncdf
 resdir <- paste(outdir,"/","VPRM_input_",domainname,"_",projinfo$year,sep="")
 nc <- create.nc(paste(resdir,"/landmask.nc",sep=""))

 # naming
 # CF standard:
 # Variable, dimension and attribute names should begin with a letter and be composed of
 # letters, digits, and underscores. 
 dim.def.nc(nc, "lon"   , dimlength=dim(map)[1])
 dim.def.nc(nc, "lat" , dimlength=dim(map)[2])

 # GLOBAL ATTRIBUTES
 # title
 att.put.nc(nc, "NC_GLOBAL", "title", "NC_CHAR", "VPRM input data for the STILT-VPRM modeling system: Land mask")
 # name of domain
 att.put.nc(nc, "NC_GLOBAL", "domain_name", "NC_CHAR", domainname)
 # institution
 att.put.nc(nc, "NC_GLOBAL", "institution", "NC_CHAR", "Max-Planck-Institute for Biogeochemistry, Jena, Germany")
 # references
 att.put.nc(nc, "NC_GLOBAL", "references", "NC_CHAR", 
 "
 Martin Jung, Kathrin Henkel, Martin Herold, and Galina Churkina.
 Exploiting synergies of global land cover products for carbon cycle modeling.
 Remote Sensing of Environment, 101(4):534-553, April 2006. 
 ")

 # cf version string
 # CF standard: 
 # We recommend that netCDF files that follow these conventions indicate this by setting the NUG
 # defined global attribute Conventions to the string value "CF-1.4" .
 att.put.nc(nc, "NC_GLOBAL", "Conventions", "NC_CHAR", "CF-1.4") 
 att.put.nc(nc, "NC_GLOBAL", "comment", "NC_CHAR", "
 Lower left grid cell corner is used to reference coordinates.
 Water is identified as gridcells with more than half of the pixels having
 SYNMAP class 0.
  ")

 var.def.nc(nc, "lat", "NC_FLOAT", 1)
 att.put.nc(nc, "lat", "long_name", "NC_CHAR", "latitude") 
 att.put.nc(nc, "lat", "units", "NC_CHAR", "degrees_north")  

 var.def.nc(nc, "lon", "NC_FLOAT", 0)
 att.put.nc(nc, "lon", "long_name", "NC_CHAR", "longitude") 
 att.put.nc(nc, "lon", "units", "NC_CHAR", "degrees_east")  

 var.def.nc(nc, "landmask",  "NC_FLOAT", c(0,1))
 att.put.nc(nc, "landmask", "units", "NC_CHAR", "count")
 att.put.nc(nc, "landmask", "long_name", "NC_CHAR", "land map based on SYNMAP")
 att.put.nc(nc, "landmask", "coordinates", "NC_CHAR", "lat lon")

 var.put.nc(nc, "landmask", map)
 var.put.nc(nc, "lat"        , lat)
 var.put.nc(nc, "lon"        , lon)

 # close file
 close.nc(nc)

