# replace values outside of relevant range ([0,1] for evi, [-1,1] for lswi) and NA in indices with MIN
replaceLZandNA <- function(indices,vari,year) {

  # get min and max, limit to ranges
  min <- getr(paste(vari,".minAggregatedClassed.",year,sep=""),datp);min[is.na(min)]<-0
  if(vari=="evi"){min[min<0]<-0;min[min>1]<-1}
  if(vari=="lswi"){min[min<- -1]<-0;min[min>1]<-1}
  max <- getr(paste(vari,".maxAggregatedClassed.",year,sep=""),datp);max[is.na(max)]<-0
  if(vari=="evi"){max[max<0]<-0;max[max>1]<-1}
  if(vari=="lswi"){max[max<- -1]<-0;max[max>1]<-1}

  for (v in 1:dim(indices)[4]) {
    for (t in 1:dim(indices)[3]) {
     if(vari=="evi")sel <- indices[,,t,v]<0 | is.na(indices[,,t,v])
     if(vari=="lswi")sel <- indices[,,t,v]< -1 | is.na(indices[,,t,v])
     indices[,,t,v][sel] <- min[,,v][sel]
     if(vari=="evi")sel2 <- indices[,,t,v]>1
     if(vari=="lswi")sel2 <- indices[,,t,v]>1
     indices[,,t,v][sel2] <- max[,,v][sel2]
    }
  }

  # replace other NAs with 0
  indices[is.na(indices)] <- 0

  return(indices)
}


writeIndices<-function(vari="evi",start_day_of_year,doyuse,resdir) {

 # make sure the needed library is loaded
 library(RNetCDF)
 
 # calculate coordinates
 lon <- seq(projinfo$x0,projinfo$x1,by=projinfo$dxy)[1:projinfo$nx]
 lat <- seq(projinfo$y0,projinfo$y1,by=projinfo$dyy)[1:projinfo$ny]
 
 # get data
 indices <- getr(paste(vari,".indicesAggregated.",projinfo$year,sep=""),datp)

 # replace NA and negative values with 0
 indices <- replaceLZandNA(indices,vari,projinfo$year)

 # file name
 # CF standard:
 # NetCDF files should have the file name extension ".nc".  
 nc <- create.nc(paste(resdir,"/VPRM_input_",toupper(vari),"_",projinfo$year,".nc",sep=""))

 # dimensions
 # CF standard:
 # If any or all of the dimensions of a variable have the interpretations
 # of "date or time" (T), "height or depth" (Z), "latitude" (Y), or "longitude" (X)
 # then we recommend, but do not require (see Section 1.4, "Relationship to the COARDS Conventions"),
 # those dimensions to appear in the relative order T, then Z, then Y, then X in the CDL definition
 # corresponding to the file. All other dimensions should, whenever possible, be placed to the left
 # of the spatiotemporal dimensions. 
 
 # have y,x,t,v
 # want v,t,y,x
 # need to put inverse order x,y,t,v
 indices <- aperm(indices, c(2,1,3,4))

 # naming
 # CF standard:
 # Variable, dimension and attribute names should begin with a letter and be composed of
 # letters, digits, and underscores. 
 dim.def.nc(nc,"lon"  , dimlength=dim(indices)[1])
 dim.def.nc(nc,"lat", dimlength=dim(indices)[2])
 dim.def.nc(nc,"time", dimlength=dim(indices)[3])
 dim.def.nc(nc,"vprm_classes", dimlength=nveg)

 # GLOBAL ATTRIBUTES
 # title
 att.put.nc(nc, "NC_GLOBAL", "title", "NC_CHAR", paste("VPRM input data for the STILT-VPRM modeling system:",toupper(vari)))
 # name of domain
 att.put.nc(nc, "NC_GLOBAL", "domain_name", "NC_CHAR", domainname)
 # institution
 att.put.nc(nc, "NC_GLOBAL", "institution", "NC_CHAR", "Max-Planck-Institute for Biogeochemistry, Jena, Germany")
 # MODIS version
 att.put.nc(nc, "NC_GLOBAL", "MODIS_version", "NC_CHAR",modis_ver)
 # preproc version
 setwd(scriptpath)
 svn_info <- system("svn info",intern=T)
 setwd(mpath)
 att.put.nc(nc, "NC_GLOBAL", "VPRMpreproc_version", "NC_CHAR", svn_info[9])
 # references
 att.put.nc(nc, "NC_GLOBAL", "references", "NC_CHAR", 
 "
 Mahadevan, P., S. Wofsy, D. M. Matross, X. Xiao, A. Dunn, J. C. Lin, C.
 Gerbig, J. W. Munger, V. Y. Chow, and E. Gottlieb, A Satellite-Based
 Biosphere Parameterization for Net Ecosystem CO2 Exchange: Vegetation
 Photosynthesis and Respiration Model (VPRM), Global Biogeochem. Cycles,
 22, GB2005, doi:10.1029/2006GB002735, 2008.

 Martin Jung, Kathrin Henkel, Martin Herold, and Galina Churkina.
 Exploiting synergies of global land cover products for carbon cycle modeling.
 Remote Sensing of Environment, 101(4):534-553, April 2006. 
 ")

 # cf version string
 # CF standard: 
 # We recommend that netCDF files that follow these conventions indicate this by setting the NUG
 # defined global attribute Conventions to the string value "CF-1.4" .
 att.put.nc(nc, "NC_GLOBAL", "Conventions", "NC_CHAR", "CF-1.4") 
 att.put.nc(nc, "NC_GLOBAL", "comment", "NC_CHAR", "
 Lower left grid cell corner is used to reference coordinates.

 SYNMAP/VPRM mapping: 
 {(1,1),(2,2),(3,3),(1,4),(2,5),(3,6),(1,7),(2,8),(3,9),(4,10),
  (4,11),(4,12),(4,13),(4,14),(4,15),(4,16),(4,17),(4,18),(5,19),(5,20),
  (5,21),(5,22),(5,23),(5,24),(5,25),(5,26),(5,27),(6,28),(6,29),(6,30),
  (6,31),(6,32),(6,33),(6,34),(6,35),(6,36),(4,37),(4,38),(4,39),(4,40),
  (7,41),(7,42),(7,43),(6,44),(8,45),(8,46),(8,47),(8,48) | (x,y) where 1<=x<=8 is VPRM class,
  1<=y<=48 is SYNMAP class}

  used VPRM classes:
  1 Trees evergreen
  2 Trees deciduous
  3 Trees mixed
  4 Trees and shrubs
  5 Trees and grasses
  6 Trees and crops
  7 grasses
  8 Barren, Urban and built-up, Permanent snow and ice  
  ")

 var.def.nc(nc, "lat", "NC_FLOAT", 1)
 att.put.nc(nc, "lat", "long_name", "NC_CHAR", "latitude") 
 att.put.nc(nc, "lat", "units", "NC_CHAR", "degrees_north")  

 var.def.nc(nc, "lon", "NC_FLOAT", 0)
 att.put.nc(nc, "lon", "long_name", "NC_CHAR", "longitude") 
 att.put.nc(nc, "lon", "units", "NC_CHAR", "degrees_east")  

 var.def.nc(nc, "start_day_of_year", "NC_INT",  NA)
 att.put.nc(nc, "start_day_of_year", "units", "NC_CHAR", paste("day of ",projinfo$year," (ordinal date)",sep=""))
 att.put.nc(nc, "start_day_of_year", "long_name", "NC_CHAR", "1st day of measurement (1st day of 1st 8 day interval)") 

 var.def.nc(nc, "time", "NC_INT", 2)
 att.put.nc(nc, "time", "units", "NC_CHAR", paste("day of ",projinfo$year," (ordinal date)",sep=""))
 att.put.nc(nc, "time", "long_name", "NC_CHAR", "8 day interval of MODIS reflectance data")

 var.def.nc(nc, "vprm_classes", "NC_BYTE", 3)
 att.put.nc(nc, "vprm_classes", "units", "NC_CHAR", "vegetation class")
 att.put.nc(nc, "vprm_classes", "long_name", "NC_CHAR", "merged SYNMAP vegetation classes")

 var.def.nc(nc, vari,  "NC_FLOAT", c(0,1,2,3))
 att.put.nc(nc, vari, "units", "NC_CHAR", "count")
 if (vari=="evi") {
  att.put.nc(nc, vari, "long_name", "NC_CHAR", "Enhanced Vegetation Index")
 } else {
  att.put.nc(nc, vari, "long_name", "NC_CHAR", "Land Surface Water Index")
 }
 att.put.nc(nc, vari, "coordniates", "NC_CHAR", "lat lon")

 var.put.nc(nc, vari         , indices)
 var.put.nc(nc, "lat"        , lat)
 var.put.nc(nc, "lon"        , lon)
 var.put.nc(nc, "start_day_of_year", start_day_of_year)
 var.put.nc(nc, "time", doyuse-projinfo$year*1000)
 var.put.nc(nc, "vprm_classes", 1:8)

 # close file
 close.nc(nc)




 ####### also write min/max indices ###############
 for (i in c("min","max")) {
  minmax <- getr(paste(vari,".",i,"AggregatedClassed.",projinfo$year,sep=""),datp)

  # replace NA with 0, and values outsite of range to min/max
  minmax[is.na(minmax)] <- 0
  if(vari=="evi"){
    minmax[minmax<0]      <- 0
    minmax[minmax>1]      <- 1
  }
  if(vari=="lswi"){
    minmax[minmax< -1]   <- -1
    minmax[minmax>1]      <- 1
  }

  minmax <- aperm(minmax, c(2,1,3))

  nc <- create.nc(paste(resdir,"/VPRM_input_",toupper(vari),"_",toupper(i),"_",projinfo$year,".nc",sep=""))

  # naming
  # CF standard:
  # Variable, dimension and attribute names should begin with a letter and be composed of
  # letters, digits, and underscores. 
  dim.def.nc(nc,"lon",dimlength=dim(minmax)[1])
  dim.def.nc(nc,"lat",dimlength=dim(minmax)[2])
  dim.def.nc(nc,"vprm_classes",dimlength=nveg)

  # GLOBAL ATTRIBUTES
  # title
  att.put.nc(nc, "NC_GLOBAL", "title", "NC_CHAR", paste("VPRM input data for the STILT-VPRM modeling system: ",toupper(vari)," (",toupper(i),")",sep=""))
  # name of domain
  att.put.nc(nc, "NC_GLOBAL", "domain_name", "NC_CHAR", domainname)
  # institution
  att.put.nc(nc, "NC_GLOBAL", "institution", "NC_CHAR", "Max-Planck-Institute for Biogeochemistry, Jena, Germany")
  # MODIS version
  att.put.nc(nc, "NC_GLOBAL", "MODIS_version", "NC_CHAR",modis_ver)
  # preproc version
  setwd(scriptpath)
  svn_info <- system("svn info",intern=T)
  setwd(mpath)
  att.put.nc(nc, "NC_GLOBAL", "VPRMpreproc_version", "NC_CHAR", svn_info[9])
  # references
  att.put.nc(nc, "NC_GLOBAL", "references", "NC_CHAR", 
  "
  Lin, J.C., C. Gerbig, S.C. Wofsy, A.E. Andrews, B.C. Daube, K.J. Davis,
  and C.A. Grainger, A near-field tool for simulating the upstream influence
  of atmospheric observations: The Stochastic Time-Inverted Lagrangian Transport
  (STILT) model, Journal of Geophysical Research-Atmospheres, 108 (D16), 2003. 

  Mahadevan, P., S. Wofsy, D. M. Matross, X. Xiao, A. Dunn, J. C. Lin, C.
  Gerbig, J. W. Munger, V. Y. Chow, and E. Gottlieb, A Satellite-Based
  Biosphere Parameterization for Net Ecosystem CO2 Exchange: Vegetation
  Photosynthesis and Respiration Model (VPRM), Global Biogeochem. Cycles,
  22, GB2005, doi:10.1029/2006GB002735, 2008.

  Martin Jung, Kathrin Henkel, Martin Herold, and Galina Churkina.
  Exploiting synergies of global land cover products for carbon cycle modeling.
  Remote Sensing of Environment, 101(4):534-553, April 2006. 
  ")

  # cf version string
  # CF standard: 
  # We recommend that netCDF files that follow these conventions indicate this by setting the NUG
  # defined global attribute Conventions to the string value "CF-1.4" .
  att.put.nc(nc, "NC_GLOBAL", "Conventions", "NC_CHAR", "CF-1.4") 
  att.put.nc(nc, "NC_GLOBAL", "comment", "NC_CHAR", "
  Lower left grid cell corner is used to reference coordinates.

  SYNMAP/VPRM mapping: 
  {(1,1),(2,2),(3,3),(1,4),(2,5),(3,6),(1,7),(2,8),(3,9),(4,10),
   (4,11),(4,12),(4,13),(4,14),(4,15),(4,16),(4,17),(4,18),(5,19),(5,20),
   (5,21),(5,22),(5,23),(5,24),(5,25),(5,26),(5,27),(6,28),(6,29),(6,30),
   (6,31),(6,32),(6,33),(6,34),(6,35),(6,36),(4,37),(4,38),(4,39),(4,40),
   (7,41),(7,42),(7,43),(6,44),(8,45),(8,46),(8,47),(8,48) | (x,y) where 1<=x<=8 is VPRM class,
   1<=y<=48 is SYNMAP class}

   used VPRM classes:
   1 Trees evergreen
   2 Trees deciduous
   3 Trees mixed
   4 Trees and shrubs
   5 Trees and grasses
   6 Trees and crops
   7 grasses
   8 Barren, Urban and built-up, Permanent snow and ice  
   ")

  var.def.nc(nc, "lat", "NC_FLOAT", 1)
  att.put.nc(nc, "lat", "long_name", "NC_CHAR", "latitude") 
  att.put.nc(nc, "lat", "units", "NC_CHAR", "degrees_north")  

  var.def.nc(nc, "lon", "NC_FLOAT", 0)
  att.put.nc(nc, "lon", "long_name", "NC_CHAR", "longitude") 
  att.put.nc(nc, "lon", "units", "NC_CHAR", "degrees_east")  

  var.def.nc(nc, "vprm_classes", "NC_BYTE", 2)
  att.put.nc(nc, "vprm_classes", "units", "NC_CHAR", "vegetation class")
  att.put.nc(nc, "vprm_classes", "long_name", "NC_CHAR", "merged SYNMAP vegetation classes")

  var.def.nc(nc, paste(vari,i,sep="_"), "NC_FLOAT", c(0,1,2))
  att.put.nc(nc, paste(vari,i,sep="_"), "units", "NC_CHAR", "count")
  if (vari=="evi") {
   att.put.nc(nc, paste(vari,i,sep="_"), "long_name", "NC_CHAR", "Enhanced Vegetation Index")
  } else {
   att.put.nc(nc, paste(vari,i,sep="_"), "long_name", "NC_CHAR", "Land Surface Water Index")
  }
  att.put.nc(nc, paste(vari,i,sep="_"), "coordniates", "NC_CHAR", "lat lon")

  var.put.nc(nc, paste(vari,i,sep="_"), minmax)
  var.put.nc(nc, "lat"        , lat)
  var.put.nc(nc, "lon"        , lon)
  var.put.nc(nc, "vprm_classes", 1:8)

  # close file
  close.nc(nc)
 }

 return(NULL)  
}





writeVegetationMap<-function(resdir) {
 
 # make sure that the needed library is loaded
 library(RNetCDF)
 
 # calculate coordinates
 lon <- seq(projinfo$x0,projinfo$x1,by=projinfo$dxy)[1:projinfo$nx]
 lat <- seq(projinfo$y0,projinfo$y1,by=projinfo$dyy)[1:projinfo$ny]

 # get data
 map <- getr(paste("mapAggregated.",projinfo$year,sep=""),datp)

 # replace NA and negative values with 0
 map[is.na(map)] <- 0
 map[map<0]      <- 0

 map<-aperm(map,c(2,1,3))

 # create netcdf file: create.ncdf
 nc <- create.nc(paste(resdir,"/VPRM_input_VEG_FRA_",projinfo$year,".nc",sep=""))

 # naming
 # CF standard:
 # Variable, dimension and attribute names should begin with a letter and be composed of
 # letters, digits, and underscores. 
 dim.def.nc(nc, "lon"   , dimlength=dim(map)[1])
 dim.def.nc(nc, "lat" , dimlength=dim(map)[2])
 dim.def.nc(nc, "vprm_classes", dimlength=nveg)

 # GLOBAL ATTRIBUTES
 # title
 att.put.nc(nc, "NC_GLOBAL", "title", "NC_CHAR", "VPRM input data for the STILT-VPRM modeling system: Vegetation Fraction Map")
 # name of domain
 att.put.nc(nc, "NC_GLOBAL", "domain_name", "NC_CHAR", domainname)
 # institution
 att.put.nc(nc, "NC_GLOBAL", "institution", "NC_CHAR", "Max-Planck-Institute for Biogeochemistry, Jena, Germany")
 # MODIS version
 att.put.nc(nc, "NC_GLOBAL", "MODIS_version", "NC_CHAR",modis_ver)
 # preproc version
 setwd(scriptpath)
 svn_info <- system("svn info",intern=T)
 setwd(mpath)
 att.put.nc(nc, "NC_GLOBAL", "VPRMpreproc_version", "NC_CHAR", svn_info[9])
 # references
 att.put.nc(nc, "NC_GLOBAL", "references", "NC_CHAR", 
 "
 Lin, J.C., C. Gerbig, S.C. Wofsy, A.E. Andrews, B.C. Daube, K.J. Davis,
 and C.A. Grainger, A near-field tool for simulating the upstream influence
 of atmospheric observations: The Stochastic Time-Inverted Lagrangian Transport
 (STILT) model, Journal of Geophysical Research-Atmospheres, 108 (D16), 2003. 

 Mahadevan, P., S. Wofsy, D. M. Matross, X. Xiao, A. Dunn, J. C. Lin, C.
 Gerbig, J. W. Munger, V. Y. Chow, and E. Gottlieb, A Satellite-Based
 Biosphere Parameterization for Net Ecosystem CO2 Exchange: Vegetation
 Photosynthesis and Respiration Model (VPRM), Global Biogeochem. Cycles,
 22, GB2005, doi:10.1029/2006GB002735, 2008.

 Martin Jung, Kathrin Henkel, Martin Herold, and Galina Churkina.
 Exploiting synergies of global land cover products for carbon cycle modeling.
 Remote Sensing of Environment, 101(4):534-553, April 2006. 
 ")

 # cf version string
 # CF standard: 
 # We recommend that netCDF files that follow these conventions indicate this by setting the NUG
 # defined global attribute Conventions to the string value "CF-1.4" .
 att.put.nc(nc, "NC_GLOBAL", "Conventions", "NC_CHAR", "CF-1.4") 
 att.put.nc(nc, "NC_GLOBAL", "comment", "NC_CHAR", "
 Lower left grid cell corner is used to reference coordinates.

 SYNMAP/VPRM mapping: 
 {(1,1),(2,2),(3,3),(1,4),(2,5),(3,6),(1,7),(2,8),(3,9),(4,10),
  (4,11),(4,12),(4,13),(4,14),(4,15),(4,16),(4,17),(4,18),(5,19),(5,20),
  (5,21),(5,22),(5,23),(5,24),(5,25),(5,26),(5,27),(6,28),(6,29),(6,30),
  (6,31),(6,32),(6,33),(6,34),(6,35),(6,36),(4,37),(4,38),(4,39),(4,40),
  (7,41),(7,42),(7,43),(6,44),(8,45),(8,46),(8,47),(8,48) | (x,y) where 1<=x<=8 is VPRM class,
  1<=y<=48 is SYNMAP class}

  used VPRM classes:
  1 Trees evergreen
  2 Trees deciduous
  3 Trees mixed
  4 Trees and shrubs
  5 Trees and grasses
  6 Trees and crops
  7 grasses
  8 Barren, Urban and built-up, Permanent snow and ice  
  ")

 var.def.nc(nc, "lat", "NC_FLOAT", 1)
 att.put.nc(nc, "lat", "long_name", "NC_CHAR", "latitude") 
 att.put.nc(nc, "lat", "units", "NC_CHAR", "degrees_north")  

 var.def.nc(nc, "lon", "NC_FLOAT", 0)
 att.put.nc(nc, "lon", "long_name", "NC_CHAR", "longitude") 
 att.put.nc(nc, "lon", "units", "NC_CHAR", "degrees_east")  

 var.def.nc(nc, "vprm_classes", "NC_BYTE", 2)
 att.put.nc(nc, "vprm_classes", "units", "NC_CHAR", "vegetation class")
 att.put.nc(nc, "vprm_classes", "long_name", "NC_CHAR", "merged SYNMAP vegetation classes")

 var.def.nc(nc, "vegetation_fraction_map",  "NC_FLOAT", c(0,1,2))
 att.put.nc(nc, "vegetation_fraction_map", "units", "NC_CHAR", "count")
 att.put.nc(nc, "vegetation_fraction_map", "long_name", "NC_CHAR", "vegetation fraction map based on SYNMAP")
 att.put.nc(nc, "vegetation_fraction_map", "coordinates", "NC_CHAR", "lat lon")

 var.put.nc(nc, "vegetation_fraction_map", map)
 var.put.nc(nc, "lat"        , lat)
 var.put.nc(nc, "lon"        , lon)
 var.put.nc(nc, "vprm_classes", 1:8)

 # close file
 close.nc(nc)

 return(NULL)
}



